﻿using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace CueTextBoxSample {
    public class CueTextBox : TextBox {

        /// <summary>
        /// Gets/sets cue text when there is no Text written.
        /// </summary>
        /// <remarks>
        /// In order for effect to be visible, EnableVisualStyles must be called first.
        /// </remarks>
        [Category("Appearance")]
        [Description("Cue text displayed in control's background before any other text is entered.")]
        [DefaultValue("")]
        public string CueText {
            get {
                var text = new StringBuilder(256);
                var res = NativeMethods.SendMessageGetCueBanner(this.Handle, NativeMethods.EM_GETCUEBANNER, text, new IntPtr(text.Capacity));
                if (res.ToInt64() != 0) {
                    return text.ToString();
                } else {
                    return null;
                }
            }
            set {
                NativeMethods.SendMessageSetCueBanner(this.Handle, NativeMethods.EM_SETCUEBANNER, new IntPtr(1), new StringBuilder(value));
            }
        }


        private class NativeMethods {

            public const UInt32 ECM_FIRST = 0x1500;
            public const UInt32 EM_SETCUEBANNER = ECM_FIRST + 1;
            public const UInt32 EM_GETCUEBANNER = ECM_FIRST + 2;


            [DllImportAttribute("user32.dll", CharSet = CharSet.Unicode, EntryPoint = "SendMessage")]
            public static extern IntPtr SendMessageSetCueBanner([In()] IntPtr hWnd,
                                                                [In()] UInt32 Msg,
                                                                [In()] IntPtr wParam,
                                                                [In()] StringBuilder lParam);

            [DllImportAttribute("user32.dll", CharSet = CharSet.Unicode, EntryPoint = "SendMessage")]
            public static extern IntPtr SendMessageGetCueBanner([In()] IntPtr hWnd,
                                                                [In()] UInt32 Msg,
                                                                [Out()] StringBuilder wParam,
                                                                [In()] IntPtr lParam);

        }

    }
}
